/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.eip;

import java.io.File;
import java.io.IOException;

import javax.xml.XMLConstants;
import javax.xml.transform.dom.DOMSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.objectweb.petals.common.descriptor.Extensions;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.su.ServiceUnitExtensionsValidator;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * Custom extensions validator
 * @version $Rev: $ $Date: $
 * @author Guillaume Decarnin - Open Wide
 */
public class EipServiceUnitExtensionsValidator implements ServiceUnitExtensionsValidator {

    private static final String META_INF = "META-INF";
    private static final String EIP_XSD_FILE_NAME = "eip-extension.xsd";

    private File xsd;

    public EipServiceUnitExtensionsValidator(String serviceEngineRootPath) {
        xsd = new File(serviceEngineRootPath + File.separator + META_INF + File.separator + EIP_XSD_FILE_NAME);
    }

    public void validateConsumesExtensions(Extensions extensions) throws PEtALSComponentSDKException {
        // nothing to do
    }

    public void validateProvidesExtensions(Extensions extensions) throws PEtALSComponentSDKException {
        try {
            Node extensionNode = extensions.getUnknownsExtensions().get(EipEngine.EIP_NAMESPACE_URI).getNode();
            validateExtensionNode(extensionNode);
        } catch (Exception e) {
            throw new PEtALSComponentSDKException(e);
        }
    }

    public void validateExtensionNode(Node extensionNode) throws SAXException, IOException {
        Validator validator = createValidator();
        validateExtensionNode(extensionNode, validator);
    }

    private Validator createValidator() throws SAXException {
        SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        Schema schemaXSD = schemaFactory.newSchema(xsd);
        return schemaXSD.newValidator();
    }

    private void validateExtensionNode(Node extensionNode, Validator validator) throws SAXException, IOException {
        validator.validate(new DOMSource(extensionNode));
    }
}
